#include "pch.hpp"
#include "Dot.hpp"
#include "Monster.hpp"

const float COLLISION_DISTANCE = 100.0f;

void Dot::Load(sf::Texture & spriteSheet, sf::Texture & glowSheet,
        sf::FloatRect const & rect, ObjectColor::Type _color)
{
    GameObject::Load(spriteSheet, 2, 10, rect);
	Animation::Sequence sequence;
    sequence.name = "shiny";
    sequence.startFrame = 0;
    sequence.endFrame = 1;
	animation.AddSequence(sequence);

    color = _color;
	animation.PlaySequence("shiny", Animation::Forward, 0);
	animation.SetScale(sf::Vector2f(1.0, 1.0));
	animation.SetRotation(0);
	animation.SetColor( color == ObjectColor::Red ? sf::Color::Red :
		(color == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue));

    glowAnim.Load(glowSheet, 2, 10);
    glowAnim.SetPosition(position);
    glowAnim.SetScale(float2(rect.width / glowAnim.GetSize().x, rect.height / glowAnim.GetSize().y));
	glowAnim.AddSequence(sequence);
	glowAnim.PlaySequence("shiny", Animation::Forward, 0);
	glowAnim.SetScale(sf::Vector2f(1.0, 1.0));
	glowAnim.SetRotation(0);
}

ObjectColor::Type Dot::GetColor(void)
{
    return color;
}

void Dot::Update()
{
    glowAnim.Update();
	animation.Update();
}

void Dot::Render()
{
    glowAnim.Render();
	GameObject::Render();
}

Dot * Dot::SpawnRandomly(sf::Texture & dotImage, sf::Texture & glowImage,
        std::vector<Monster *> const & monsters, Player & player, std::vector<Dot *> const & dots)
{
    Dot * dot = new Dot;
    bool noCollision;
    float2 pos;
    do
    {
        pos = float2(float(55 + rand() % (window.getSize().x - 140)), float(60 + rand() % (window.getSize().y - 120)));
        noCollision = true;
        for (uint i = 0 ; i < monsters.size() ; ++i)
        {
            if (sfm::Length(monsters[i]->GetAnimation().GetPosition() - pos) <= COLLISION_DISTANCE)
            {
                noCollision = false;
                break;
            }
        }
        if (sfm::Length(player.GetPosition() - pos) <= 2.0f * COLLISION_DISTANCE)
        {
            noCollision = false;
        }
    }
    while (!noCollision);
    dot->Load(dotImage, glowImage, sf::FloatRect(pos.x, pos.y,
        (float)dotImage.getSize().x / 2, (float)dotImage.getSize().y), ObjectColor::Type(rand() % ObjectColor::Count));
    return dot;
}
