#include "pch.hpp"
#include "GameScreen.hpp"

const uint DOT_COUNT = 15;
const uint MONSTER_COUNT = 6;
const float STRIPES_SPEED = 5.0f;
const float STRIPES_TIME = 4.0f;
const float FLICKER_TIME = 10.0f;
const float FLICKERING_TIME = 0.7f;
const uint STRIPE_COUNT = 3;
const float PLAYER_IMMORTALITY_TIME = 3.0f;

void GameScreen::StartStripes(void)
{
    stripeDirection = rand() % 4;
    stripesTimer = STRIPES_TIME;
    uint currColor = (lastColor + 2) % ObjectColor::Count;
    for (uint i = 0 ; i < STRIPE_COUNT ; ++i)
    {
        switch (stripeDirection)
        {
            case 0:   // left-top corner
                stripes[i].setPosition(-50.0f - i * 50.0f / 1.4142f, -50.0f - i * 50.0f / 1.4142f);
                stripes[i].setRotation(45.0f);
                break;
            case 1:   // right-top corner
                stripes[i].setPosition(750.0f + i * 50.0f / 1.4142f, -50.0f - i * 50.0f / 1.4142f);
                stripes[i].setRotation(135.0f);
                break;
            case 2:   // right-bottom corner
                stripes[i].setPosition(750.0f + i * 50.0f / 1.4142f, 950.0f + i * 50.0f / 1.4142f);
                stripes[i].setRotation(225.0f);
                break;
            case 3:   // left-bottom corner
                stripes[i].setPosition(-50.0f - i * 50.0f / 1.4142f, 950.0f + i * 50.0f / 1.4142f);
                stripes[i].setRotation(315.0f);
                break;
        }
        stripes[i].setFillColor(currColor == ObjectColor::Red ? sf::Color::Red
            : currColor == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue);
        ++currColor;
        currColor %= ObjectColor::Count;
    }
	whooshSound.Play();
}

void GameScreen::SetScoreText(void)
{
    std::stringstream format;
    format << "Score: " << score;
    std::string out;
    std::getline(format, out);
    scoreText.setString(out);
}

void GameScreen::Load(void)
{
    phreakFont.loadFromFile("data/VideoPhreak.ttf");
	bool result;
	result = dotImage.loadFromFile("data/dot.png");
    assert(result);
	result = glowImage.loadFromFile("data/dot_glow.png");
    assert(result);
	result = monsterImage.loadFromFile("data/monster.png");
    assert(result);
	result = guiImage.loadFromFile("data/gui800_600.png");
	assert(result);
	whooshSound.Load("data/whoosh.wav");
	popSound.Load("data/pop.aif");
	lifeLostSound.Load("data/lifelost.wav");
	result = normalMusic.openFromFile("data/8_bit.ogg");
    assert(result);
    normalMusic.setLoop(true);
	result = comboMusic.openFromFile("data/boss.wav");
    assert(result);
    comboMusic.setLoop(true);
    background.setSize(sf::Vector2f(window.getSize()));
    background.setPosition(sf::Vector2f(0.0f, 0.0f));
    background.setFillColor(sf::Color(255, 255, 255));
	result = playerImage.loadFromFile("data/player1.png");
    assert(result);
	player.Load(playerImage, sf::FloatRect(56.0f, 352.0f, 44.0f, 56.0f));
    PopupText::Params params;
    params.fontName = "data/VideoPhreak.ttf";
    params.size = 35;
    params.style = sf::Text::Italic;
    params.color = sf::Color(0, 0, 0);
    params.duration = 1.0f;
    params.distance = 30.0f;
    params.alphaFade = true;
    popupText.Load(params);
	result = life.loadFromFile("data/life.png");
    assert(result);
    for (uint i = 0 ; i < 3 ; ++i)
    {
        lives[i].setTexture(life);
		lives[i].setScale(0.45f, 0.45f);
        lives[i].setPosition(120.0f + 55.0f * i, 23.0f);
        lives[i].setOrigin(life.getSize().x * 0.5f, life.getSize().y * 0.5f);
    }
	gui.setTexture(guiImage);
	gui.setPosition(0, 0);
}

void GameScreen::Init(void)
{
    scoreText.setFont(phreakFont);
    scoreText.setString("Score: 0");
    scoreText.setCharacterSize(24);
    scoreText.setColor(sf::Color::White);
    scoreText.setStyle(sf::Text::Bold);
    scoreText.setPosition(530.0f, 8.0f);

	player.Init(float2(56.0f, 352.0f));

    for (uint i = 0 ; i < dots.size() ; ++i)
    {
        delete dots[i];
    }
    dots.clear();
    for (uint i = 0 ; i < monsters.size() ; ++i)
    {
        delete monsters[i];
    }
    monsters.clear();

    dots.resize(DOT_COUNT);
    for (uint i = 0 ; i < dots.size() ; ++i)
    {
        dots[i] = Dot::SpawnRandomly(dotImage, glowImage, monsters, player, dots);
    }
    monsters.resize(MONSTER_COUNT);
    for (uint i = 0 ; i < monsters.size() ; ++i)
    {
        monsters[i] = Monster::SpawnRandomly(monsterImage, dots, player);
    }

    lastColor = ObjectColor::Count;
    colorAccum = 0.0f;
    playerLives = 3;

    for (uint i = 0 ; i < STRIPE_COUNT ; ++i)
    {
        stripes[i].setSize(float2(50.0f, 10000.0f));
        stripes[i].setPosition(float2(-200.0f, 0.0f));
        stripes[i].setOrigin(25.0f, 5000.0f);
    }
    stripesTimer = 0.0f;
    stripesBonusAvailable = false;
    backgroundFlickerTimer = 0.0f;
    flickeringTimer = 0.0f;

    playerImmortalityTimer = 0.0f;

    score = 0;
    comboCount = 0;

    popupText.Init();

    background.setFillColor(sf::Color::White);

    normalMusic.play();
}

void GameScreen::HandleEvent(sf::Event const & event)
{
}

void GameScreen::Update()
{
	player.Update();

	for (uint i = 0 ; i < monsters.size() ; ++i)
	{
		if(lastColor == monsters[i]->GetColor() && colorAccum == 255)
			monsters[i]->SetVisible(false);
		if(monsters[i]->GetVisible() == false && lastColor != monsters[i]->GetColor())
			monsters[i]->SetVisible(true);
		monsters[i]->Update(player.GetPosition());

		for(uint j = 0; j < dots.size(); ++j)
		{
			if(dots[j]->GetRect().intersects(monsters[i]->GetRect()))
			{
				monsters[i]->SetColor(dots[j]->GetColor());

				delete dots[j];
				dots[j] = Dot::SpawnRandomly(dotImage, glowImage, monsters, player, dots);
			}
		}
    }

    if (playerImmortalityTimer == 0.0f)
    {
	    for (uint i = 0 ; i < monsters.size() ; ++i)
	    {
            if (monsters[i]->GetRect().intersects(player.GetRect()))
            {
                --playerLives;
                playerImmortalityTimer = PLAYER_IMMORTALITY_TIME;
                auto color = monsters[i]->GetColor();
                ObjectColor::Type newColor;
                do
                {
                    newColor = ObjectColor::Type(rand() % ObjectColor::Count);
                }
                while (color == newColor);
                monsters[i]->SetColor(newColor);
                if (playerLives == 0)
                {
                    gameScore = score;
                    screenManager.SetCurrentScreen((Screen &)resultScreen);
                    screenManager.Init();
                    normalMusic.stop();
                    comboMusic.stop();
                }
                popupText.Show("Aaaaaaaaa!", float2(player.GetRect().left, player.GetRect().top));
                lifeLostSound.Play();
                break;
            }
	    }
    }
    else
    {
        playerImmortalityTimer -= ONE_SIXTIETH;
        if (playerImmortalityTimer <= 0.0f)
        {
            playerImmortalityTimer = 0.0f;
        }
        float alpha = std::fmod(playerImmortalityTimer, .50f);
        if (alpha > 0.25f)
        {
            alpha = .50f - alpha;
        }
        alpha *= 4.0f;
        alpha = 1.0f - alpha;
        player.SetAlpha(alpha);
    }

    if (stripesTimer > 0.0f)
    {
        stripesTimer -= ONE_SIXTIETH;
        if (stripesTimer <= 0.0f)
        {
            StartStripes();
        }
    }

    if (stripesBonusAvailable)
    {
        for (uint i = 0 ; i < STRIPE_COUNT ; ++i)
        {
            auto pos = stripes[i].getPosition();
            switch (stripeDirection)
            {
                case 0:   // left-top corner
                pos += float2(STRIPES_SPEED, STRIPES_SPEED);
                    break;
                case 1:   // right-top corner
                pos += float2(-STRIPES_SPEED, STRIPES_SPEED);
                    break;
                case 2:   // right-bottom corner
                pos += float2(-STRIPES_SPEED, -STRIPES_SPEED);
                    break;
                case 3:   // left-bottom corner
                pos += float2(STRIPES_SPEED, -STRIPES_SPEED);
                    break;
            }
            stripes[i].setPosition(pos);
        }

        if (backgroundFlickerTimer > 0.0f)
        {
            backgroundFlickerTimer -= ONE_SIXTIETH;
            if (backgroundFlickerTimer <= 0.0f)
            {
                flickeringTimer = 2.0f * FLICKERING_TIME;
                uint currColor = (lastColor + 1) % ObjectColor::Count;
                background.setFillColor(currColor == ObjectColor::Red ? sf::Color::Red
                    : currColor == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue);
                backgroundFlickerTimer = FLICKER_TIME;
            }
        }

        if (flickeringTimer > 0.0f)
        {
            flickeringTimer -= ONE_SIXTIETH;
            if (flickeringTimer <= FLICKERING_TIME + ONE_SIXTIETH && flickeringTimer > FLICKERING_TIME)
            {
                uint currColor = (lastColor + 2) % ObjectColor::Count;
                background.setFillColor(currColor == ObjectColor::Red ? sf::Color::Red
                    : currColor == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue);
            }
            else if (flickeringTimer <= 0.0f)
            {
                flickeringTimer = 0.0f;
                background.setFillColor(lastColor == ObjectColor::Red ? sf::Color::Red
                    : lastColor == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue);
            }
        }
    }

    for (uint i = 0 ; i < dots.size() ; ++i)
    {
		dots[i]->Update();
        if (player.GetRect().intersects(dots[i]->GetRect()))
        {
			ObjectColor::Type color = dots[i]->GetColor();
			sf::Color bgColor;

            if (color != lastColor)
            {
                colorAccum = colorStep;
                stripesBonusAvailable = false;
                backgroundFlickerTimer = 0.0f;
                flickeringTimer = 0.0f;
                stripesTimer = 0.0f;
                comboCount = 1;
                lastPoints = 1.0f;

                if (comboMusic.getStatus() == sf::SoundSource::Playing)
                {
                    comboMusic.pause();
                    normalMusic.play();
                }
            }
            else
            {
                if (colorAccum < 255)
                {
                    colorAccum += colorStep;
                    if (colorAccum >= 255)
                    {
                        colorAccum = 255;
                        stripesBonusAvailable = true;

                        comboMusic.play();
                        normalMusic.pause();

                        StartStripes();
                        backgroundFlickerTimer = FLICKER_TIME;
                    }
                }
                ++comboCount;
            }
            float base = 1.0f + std::pow(1.01f, -80.0f - 2.0f * comboCount);
            float points = base * lastPoints;
            lastPoints = points;
            score += (uint)points;
            std::stringstream format;
            format << '+' << (uint)points;
            std::string pointsText;
            format >> pointsText;
            popupText.Show(pointsText, float2(player.GetRect().left, player.GetRect().top));
            popSound.Play();

            lastColor = color;
			switch(color)
			{
				case ObjectColor::Red:
					bgColor.r = 255;
					bgColor.g = 255 - (uint)colorAccum;
					bgColor.b = 255 - (uint)colorAccum;
					break;
				case ObjectColor::Green:
					bgColor.r = 255 - (uint)colorAccum;
					bgColor.g = 255;
					bgColor.b = 255 - (uint)colorAccum;
					break;
				case ObjectColor::Blue:
					bgColor.r = 255 - (uint)colorAccum;
					bgColor.g = 255 - (uint)colorAccum;
					bgColor.b = 255;
					break;
			}

			delete dots[i];
            dots[i] = Dot::SpawnRandomly(dotImage, glowImage, monsters, player, dots);

			background.setFillColor(bgColor);
        }
    }

    SetScoreText();

	whooshSound.Update();
    popSound.Update();
    lifeLostSound.Update();

    popupText.Update();
}

void GameScreen::Render(void)
{
    // background
    window.draw(background);
    if (stripesBonusAvailable)
    {
        for (uint i = 0 ; i < STRIPE_COUNT ; ++i)
        {
            window.draw(stripes[i]);
        }
    }

    // game objects
    for (uint i = 0 ; i < dots.size() ; ++i)
    {
        dots[i]->Render();
    }
	player.Render();
    for (uint i = 0 ; i < monsters.size() ; ++i)
    {
        monsters[i]->Render();
    }
    popupText.Render();

    // gui
	window.draw(gui);
    for (uint i = 0 ; i < 3 ; ++i)
    {
        if (i < playerLives)
        {
            window.draw(lives[i]);
        }
    }
    window.draw(scoreText);
}

void GameScreen::Unload(void)
{
    for (uint i = 0 ; i < dots.size() ; ++i)
    {
        delete dots[i];
    }
    for (uint i = 0 ; i < monsters.size() ; ++i)
    {
        delete monsters[i];
    }
}
