#include "pch.hpp"
#include "IntroScreen.hpp"

const float INTRO_TIME = 6.0f;
const float FADE_IN_TIME = 0.05f;

void IntroScreen::Load(void)
{
	bool result;
    result = candyImage.loadFromFile("data/candy.png");
	assert(result);
	result = stickImage.loadFromFile("data/stick.png");
	assert(result);
	result = font.loadFromFile("data/VideoPhreak.ttf");
	assert(result);
    candySprite.setTexture(candyImage);
    candySprite.setOrigin(candyImage.getSize().x * 0.5f, candyImage.getSize().y * 0.5f);
    candySprite.setPosition(window.getSize().x * 0.5f, window.getSize().y * 0.5f);
	candySprite.setScale(4, 4);
	candySprite.setColor(sf::Color(255,255,255,0));
	stickSprite.setTexture(stickImage);
	stickSprite.setScale(4, 4);
	stickSprite.setColor(sf::Color(255,255,255,0));
	stickSprite.setPosition(candySprite.getPosition().x + 22, candySprite.getPosition().y + 32);

	candyText.setFont(font);
	candyText.setCharacterSize(80);
	candyText.setPosition(150, 400);
	candyText.setString("CANDY");
	candyText.setColor(sf::Color(255,255,255,0));

    candySound.Load("data/candy.wav");
}

void IntroScreen::Init(void)
{
    introTimer = INTRO_TIME;
	fadeTimer = FADE_IN_TIME;

    candySound.Play();
}

void IntroScreen::HandleEvent(sf::Event const & event)
{
}

void IntroScreen::Update()
{
    introTimer -= ONE_SIXTIETH;
	fadeTimer -= ONE_SIXTIETH;
    if (introTimer <= 0.0f)
    {
        screenManager.SetCurrentScreen((Screen &)menuScreen);
        screenManager.Init();
    }
	if(fadeTimer <= 0.0f && candySprite.getColor().a < 255)
	{
		candySprite.setColor(sf::Color(255,255,255, (candySprite.getColor().a + 10 >= 255 ? 255 : candySprite.getColor().a + 10)));
		fadeTimer = FADE_IN_TIME;
	}
	if(candySprite.getColor().a == 255 && candySprite.getPosition().x > 350)
	{
		candySprite.move(-1.0,-1.0);
		stickSprite.setColor(sf::Color(255,255,255, (stickSprite.getColor().a + 10 >= 255 ? 255 : stickSprite.getColor().a + 10)));
		candyText.setColor(sf::Color(255,255,255, (candyText.getColor().a + 10 >= 255 ? 255 : candyText.getColor().a + 10)));
		candySprite.rotate(-1.0);
	}
	if(introTimer <= 1.0f)
	{
		candySprite.setColor(sf::Color(255,255,255, (candySprite.getColor().a - 10 <= 0 ? 0 : candySprite.getColor().a - 10)));
		stickSprite.setColor(sf::Color(255,255,255, (stickSprite.getColor().a - 10 <= 0 ? 0 : stickSprite.getColor().a - 10)));
		candyText.setColor(sf::Color(255,255,255, (candyText.getColor().a - 10 <= 0 ? 0 : candyText.getColor().a - 10)));
	}
	candySprite.rotate(1.5);

    candySound.Update();
}

void IntroScreen::Render(void)
{
    window.draw(stickSprite);
    window.draw(candySprite);
	window.draw(candyText);
}

void IntroScreen::Unload(void)
{
}