#include "pch.hpp"
#include "Monster.hpp"
#include <cstdlib>
#include <ctime>

const float MONSTER_SPEED = 1.0f;
const float COLLISION_DISTANCE = 100.0f;

void Monster::Load(sf::Texture & spriteSheet, sf::FloatRect const & rect, ObjectColor::Type _color)
{
    GameObject::Load(spriteSheet, 6, 10, rect);
    Animation::Sequence sequence;
    sequence.name = "idle";
    sequence.startFrame = 0;
    sequence.endFrame = 5;

    animation.AddSequence(sequence);
	animation.PlaySequence("idle", Animation::Forward, 0);
    color = _color;
    animation.SetColor(color == ObjectColor::Red ? sf::Color::Red :
		(color == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue));

    animation.SetRotation(0.0f);

	isVisible = true;
	isMoving = false;
}

void Monster::Update(float2 playerPos)
{
    GameObject::Update();

	if(!isMoving && isVisible)
	{
		float minX = 40.0, maxX = 800 - 40;
		float minY = 50.0, maxY = 600 - 25.0;

		targetPos.x = minX + static_cast <float> (rand()) /( static_cast <float> (RAND_MAX/(maxX-minX)));
		targetPos.y = minY + static_cast <float> (rand()) /( static_cast <float> (RAND_MAX/(maxY-minY)));

		isMoving = true;
	}

	if(!isVisible)
		targetPos = playerPos;

    auto dir = targetPos - position;
    if (sfm::Length(dir) <= MONSTER_SPEED)
    {
        position = targetPos;
		isMoving = false;
    }
    else
    {
		if(dir.x > 0)
			animation.SetScale(sf::Vector2f(-1, 1));
		else
			animation.SetScale(sf::Vector2f(1, 1));

        sfm::Normalize(dir);
        position += dir * MONSTER_SPEED;
    }
    animation.SetPosition(position);

    rect.left = position.x - animation.GetSize().x * 0.5f;
    rect.top = position.y - animation.GetSize().y * 0.5f;
}

ObjectColor::Type Monster::GetColor(void)
{
    return color;
}

void Monster::SetColor(ObjectColor::Type _color)
{
    color = _color;
    animation.SetColor(color == ObjectColor::Red ? sf::Color::Red :
		(color == ObjectColor::Green ? sf::Color::Green : sf::Color::Blue));
}

Monster * Monster::SpawnRandomly(sf::Texture & monsterImage, std::vector<Dot *> const & dots,
    Player & player)
{
    Monster * monster = new Monster;
    bool noCollision;
    float2 pos;
    do
    {
        pos = float2(float(55 + rand() % (window.getSize().x - 140)), float(60 + rand() % (window.getSize().y - 120)));
        noCollision = true;
        for (uint i = 0 ; i < dots.size() ; ++i)
        {
            if (sfm::Length(dots[i]->GetAnimation().GetPosition() - pos) <= COLLISION_DISTANCE)
            {
                noCollision = false;
                break;
            }
        }
        if (sfm::Length(player.GetPosition() - pos) <= 2.0f * COLLISION_DISTANCE)
        {
            noCollision = false;
        }
    }
    while (!noCollision);
    monster->Load(monsterImage, sf::FloatRect(pos.x, pos.y,
        41.0f, 51.0f), ObjectColor::Type(rand() % ObjectColor::Count));
    return monster;
}

void Monster::SetVisible(bool isVisible)
{
	this->isVisible = isVisible;

	if(isVisible)
		isMoving = false;
}

bool Monster::GetVisible()
{
	return isVisible;
}
