#include "pch.hpp"
#include "Player.hpp"

void Player::Init(float2 initPos)
{
    position = initPos;
	lastPosition = float2(0,0);
}

void Player::Load(sf::Texture & spriteSheet, sf::FloatRect const & rect)
{
    GameObject::Load(spriteSheet, 8, 5, rect);
    Animation::Sequence sequence;
    sequence.name = "walk";
    sequence.startFrame = 0;
    sequence.endFrame = 7;
    animation.AddSequence(sequence);
    sequence.name = "idle";
    sequence.startFrame = 3;
    sequence.endFrame = 3;
    animation.AddSequence(sequence);
    animation.PlaySequence("idle", Animation::Forward, 0);
    lastMousePos = GetMousePos();
	lastPosition = float2(0,0);
    wasMousePressed = false;
    playerGrabbed = false;
	wasWalking = false;
}

void Player::Update()
{
	if(sf::Joystick::isConnected(0))
	{
		float x = abs(sf::Joystick::getAxisPosition(0, sf::Joystick::X) * 0.05f) > 0.0001 ? sf::Joystick::getAxisPosition(0, sf::Joystick::X) * 0.05f : 0;
		float y = abs(sf::Joystick::getAxisPosition(0, sf::Joystick::Y) * 0.05f) > 0.0001 ? sf::Joystick::getAxisPosition(0, sf::Joystick::Y) * 0.05f : 0;

		position += sf::Vector2f(x,y);
	}

    float2 mousePos = GetMousePos();
    if (sf::Mouse::isButtonPressed(sf::Mouse::Left))
    {
        if (!wasMousePressed && rect.contains(mousePos))
        {
            playerGrabbed = true;
        }
        if (playerGrabbed)
        {
			position = mousePos;
        }
    }
    else
    {
        playerGrabbed = false;
    }
	wasMousePressed = (!wasMousePressed && playerGrabbed) ? true : false;

	if(lastPosition.x != position.x || lastPosition.y != position.y)
	{
		if(position.x < 40.0)
			position.x = 40.0;
		else if(position.x > 800.0 - 40.0)
			position.x = 800.0 - 40.0;
		if(position.y < 50.0)
			position.y = 50.0;
		else if(position.y > 600.0 - 25.0)
			position.y = 600.0 - 25.0;

		animation.SetPosition(position);
		rect.left = position.x - rect.width / 2;
		rect.top = position.y - rect.height / 2;

		if(position.x - lastPosition.x < 0)
			animation.SetScale(float2(-1, 1));
		else
			animation.SetScale(float2(1,1));
		if(!wasWalking)
		{
			animation.PlaySequence("walk", Animation::Forward, 0);
			wasWalking = true;
		}
	}
	else if(wasWalking)
	{
		animation.PlaySequence("idle", Animation::Forward, 0);
		wasWalking = false;
	}
	lastPosition = position;

	animation.Update();
}

void Player::Render()
{
	animation.Render();
}

float2 Player::GetPosition()
{
	return position;
}

void Player::SetAlpha(float alpha)
{
    sf::Color color = sf::Color::White;
    color.a = uint(255.0f * alpha);
    animation.SetColor(color);
}
