#include "pch.hpp"
#include "IntroScreen.hpp"
#include "MenuScreen.hpp"
#include "CreditsScreen.hpp"
#include "TutorialScreen.hpp"
#include "GameScreen.hpp"
#include "ResultScreen.hpp"

sf::RenderWindow window;
ScreenManager screenManager;
IntroScreen introScreen;
MenuScreen menuScreen;
CreditsScreen creditsScreen;
TutorialScreen tutorialScreen;
GameScreen gameScreen;
ResultScreen resultScreen;
const float ONE_SIXTIETH = 1.0f / 60.0f;
uint gameScore = 0;

int main(void)
{
#ifdef _DEBUG
	window.create(sf::VideoMode(800, 600, 32), "SFML Framework", sf::Style::Titlebar | sf::Style::Close);
#else
	window.create(sf::VideoMode(800, 600, 32), "SFML Framework", sf::Style::Titlebar | sf::Style::Close | sf::Style::Fullscreen);
#endif
    window.setVerticalSyncEnabled(true);

    std::srand((uint)std::time(NULL));

    screenManager.AddScreen(introScreen);
    screenManager.AddScreen(menuScreen);
    screenManager.AddScreen(creditsScreen);
    screenManager.AddScreen(tutorialScreen);
    screenManager.AddScreen(gameScreen);
    screenManager.AddScreen(resultScreen);
    screenManager.SetCurrentScreen(introScreen);
    screenManager.Load();
    screenManager.Init();

    sf::Image icon;
	bool result = icon.loadFromFile("data/candy_icon.png");
    assert(result);
    window.setIcon(32, 32, icon.getPixelsPtr());

    sf::Clock clock;
    float dt = 0.0f;
    float lastUpdateTime = clock.getElapsedTime().asSeconds();
    float accumulator = 0.0f;
    const float MAX_ACCUMULATED_TIME = 1.0f;
    while (window.isOpen())
    {
        dt = clock.getElapsedTime().asSeconds() - lastUpdateTime;
        lastUpdateTime += dt;
        dt = std::max(0.0f, dt);
        accumulator += dt;
        accumulator = Clamp(accumulator, 0.0f, MAX_ACCUMULATED_TIME);
        sf::Event event;
        while (window.pollEvent(event))
        {
            if (event.type == sf::Event::Closed)
            {
                window.close();
            }
            else
            {
                screenManager.HandleEvent(event);
            }
        }
        while (accumulator > ONE_SIXTIETH)
        {
            screenManager.Update();
            accumulator -= ONE_SIXTIETH;
        }
        window.clear(sf::Color(0, 0, 0));
        screenManager.Render();
        window.display();
    }
    screenManager.Unload();
    return EXIT_SUCCESS;
}